/////////////////////////////////////////////////////////////////////////////
// NAME: ioctrl.c
// Funktion: gibt alle 10 Sekunden die Daten von 6 Maxim DS1820 (Temperatur-
// sensor) im csv-Format ber die serielle Schnittstelle aus. An diese ist
// ein RS-232 auf WLAN Adapter (WiPort von Lantronix) angeschlossen, mit des-
// sen Hilfe die Daten in das lokale Netz bzw. in das Internet eingespeist
// werden (Telnet Protokoll). Zustzlich werden die 6 Temperaturen auch noch
// auf einem angeschlossenen LCD-Display angezeigt. ber die RS-232 werden
// zustzlich zwei "virtuelle" Temperaturen mitgesendet. Die erste davon
// zeigt den Status der Umwlzpumpe der Gastherme an (wurde einfach mit ei-
// nem zur Pumpe parallelgeschalteten 230V Relais abgenommen) und zwar 20
// Grad fr Pumpe AUS und 40 Grad fr Pumpe EIN (siehe main()). Die zweite
// virtuelle Temperatur zeigt den Brennerstand an, 25 Grad = Brenner AUS,
// 39,8 Grad = Brenner EIN. Damit kann ich im Excel-Diagramm zusammen mit
// den Temperaturkurven auch die EIN/AUS-Stati von Pumpe und Brenner an-
// zeigen. Die Brennerflamme wurde brigens einfach mit einem Fototransistor
// dedektiert, dessen Signal vom Analogkomparator der ATmega16 ausgewertet
// wird (siehe main() ganz unten).
//
// Der Datenlogger wurden im Feber 2005 dazu verwendet, die Temperatur der
// Kche, des Vorlaufs und der vier Rcklufe der Heizungstherme aufzuzeich-
// nen und die Heizung zu optimieren, was letztendlich auch zur vollsten Zu-
// friedenheit gelungen ist :-).
//
// Als Hardware fr den Logger wurde brigens ein "briggebliebenes" STK200
// AVR-Evaluationboard von Kanda verwendet.
//
// Version: 1.00
// Start: 01.02.2005
// Author: Erwin Baeck                     
// Chip: ATmega16
// Clock: 16 MHz
/////////////////////////////////////////////////////////////////////////////

#include <mega16.h>
#include <string.h>
#include <delay.h>
#include <stdio.h>
#include <math.h>

// 1-Wire port definition
#asm
.equ __w1_port = 0x15
.equ __w1_bit = 5
#endasm

#include <ds1820.h>
#include <1wire.h>

#include "ioctrl.h"
#include "strings.h"
#include "lcd.h"
#include "irq.h"

#define MAX_DEVICES 8

eeprom UBYTE dummy = 0;

//---------------------------------------------------------------------------
// Start des Programms
//---------------------------------------------------------------------------
void main(void)
{
UBYTE rom_code[MAX_DEVICES,9];
UBYTE gLineBuffer[42];
UBYTE gLCDBuffer[6];
UBYTE devices = 0;
//UBYTE counter = 0;
UWORD temp;
UBYTE tmp;
UBYTE i,j;

//---------------------------------------------------------------------------
// Port A initialization
//
// PORTA.0 = RS		44780-LCD
// PORTA.1 = R/-W	44780-LCD
// PORTA.2 = E		44780-LCD
// PORTA.4 = D4		44780-LCD
// PORTA.5 = D5		44780-LCD
// PORTA.6 = D6		44780-LCD
// PORTA.7 = D7		44780-LCD
//
//---------------------------------------------------------------------------
PORTA = 0b11111111;
DDRA  = 0b11110111;

//---------------------------------------------------------------------------
// Port B initialization
//
// PORTB.0 = XCK/T0 = Umwlzpumpenrelais IN
// PORTB.1 = T1
// PORTB.2 = INT2/AIN0
// PORTB.3 = OC0/AIN1
// PORTB.4 = -SS
// PORTB.5 = MOSI
// PORTB.6 = MISO
// PORTB.7 = SCK
//---------------------------------------------------------------------------
PORTB = 0b11110011;
DDRB  = 0b00000000;

//---------------------------------------------------------------------------
// Port C initialization
//
// PORTC.0 = SCL
// PORTC.1 = SDA
// PORTC.2 = TCK
// PORTC.3 = TMS
// PORTC.4 = TD0
// PORTC.5 = TDI = 1-Wire Port
// PORTC.6 = TOSC1
// PORTC.7 = TOSC2
//---------------------------------------------------------------------------
PORTC = 0b11111111;
DDRC  = 0b00100000;

//---------------------------------------------------------------------------
// Port D initialization
//
// PORTD.0 = RXD
// PORTD.1 = TXD
// PORTD.2 = INT0
// PORTD.3 = INT1
// PORTD.4 = OC1B
// PORTD.5 = OC1A
// PORTD.6 = ICP
// PORTD.7 = OC2
//---------------------------------------------------------------------------
PORTD = 0b11111111;
DDRD  = 0b10000000;

//---------------------------------------------------------------------------
// Timer/Counter 0 initialization
// Clock source: System Clock
// Clock value: 250,000 kHz
// Mode: Normal top=FFh
// OC0 output: Disconnected
//---------------------------------------------------------------------------
TCCR0 = 0x03;
TCNT0 = 0x00;
OCR0  = 0x00;

//---------------------------------------------------------------------------
// Timer/Counter 1 initialization
// Clock source: System Clock
// Clock value: 16000,000 kHz
// Mode: Normal top=FFFFh
// OC1A output: Discon.
// OC1B output: Discon.
// Noise Canceler: Off
// Input Capture on Falling Edge
//---------------------------------------------------------------------------
TCCR1A = 0x00;
TCCR1B = 0x01;
TCNT1H = 0x00;
TCNT1L = 0x00;
OCR1AH = 0x00;
OCR1AL = 0x00;
OCR1BH = 0x00;
OCR1BL = 0x00;

//---------------------------------------------------------------------------
// Timer/Counter 2 initialization
// Clock source: System Clock
// Clock value: 16000,000 kHz
// Mode: Normal top = FFh
// OC2 output: Disconnected
//---------------------------------------------------------------------------
ASSR = 0x00;
TCCR2 = 0x01;
TCNT2 = 0x00;
OCR2 = 0x00;

//---------------------------------------------------------------------------
// External Interrupt(s) initialization
// INT0(PD2): Off
// INT1(PD3): Off
// INT2(PB2): Off
//---------------------------------------------------------------------------
GICR  |= 0b00000000;
MCUCR  = 0b00000000;
MCUCSR = 0b00000000;
GIFR   = 0b01000000;

//---------------------------------------------------------------------------
// Timer(s)/Counter(s) Interrupt(s) initialization
// T0 OVL: on
//---------------------------------------------------------------------------
TIMSK = 0b00000001;

//---------------------------------------------------------------------------
// USART initialization
// Communication Parameters: 8 Data, 1 Stop, No Parity
// USART Receiver: On
// USART Transmitter: On
// USART Mode: Asynchronous
// USART Baud rate: 38400
//---------------------------------------------------------------------------
UCSRA = 0x00;
UCSRB = 0x18;
UCSRC = 0x86;
UBRRH = 0x00;	// 38400 Baud fr WiPort
UBRRL = 0x19;	// 38400 Baud fr WiPort

//---------------------------------------------------------------------------
// Analog Comparator initialization
// Analog Comparator: Off
// Analog Comparator Input Capture by Timer/Counter 1: Off
// Analog Comparator Output: Off
//---------------------------------------------------------------------------
ACSR = 0x80;
SFIOR = 0x00;

//---------------------------------------------------------------------------
// ADC initialization
// ADC Clock frequency: 125,000 kHz
// ADC Voltage Reference: AREF pin
// ADC High Speed Mode: Off
// ADC Auto Trigger Source: Free Running
//---------------------------------------------------------------------------
ADMUX	= 0b00000000;	//use external reference + ADC3
ADCSRA	= 0b10000111;	//ADC enabled, no IRQ, clk/128
SFIOR  &= 0b11101111;	//fast conversion off

//---------------------------------------------------------------------------
// SPI initialization
// SPI Type: none
// SPI Clock Rate: 4000,000 kHz
// SPI Clock Phase: Cycle Half
// SPI Clock Polarity: Low
// SPI Data Order: MSB First
//---------------------------------------------------------------------------
SPCR = 0x00;
SPSR = 0x00;

//---------------------------------------------------------------------------
// 2 Wire Bus initialization
// Generate Acknowledge Pulse: Off
// 2 Wire Bus Slave Address: 0h
// General Call Recognition: Off
// Bit Rate: 100,000 kHz
//---------------------------------------------------------------------------
TWBR = 0x48;
TWAR = 0x00;
TWCR = 0x04;

//---------------------------------------------------------------------------
// Analog Comparator initialization
// Analog Comparator: On
// Analog Comparator Input Capture by Timer/Counter 1: Off
//---------------------------------------------------------------------------
ACSR  = 0x00;
SFIOR = 0x00;


//---------------------------------------------------------------------------
// Clear the SPI interrupt flag
//---------------------------------------------------------------------------
#asm
in   r30,spsr
in   r30,spdr
#endasm

//---------------------------------------------------------------------------
// Display initialisieren
//---------------------------------------------------------------------------
lcd_init(16);

//---------------------------------------------------------------------------
// Resettext anzeigen
//---------------------------------------------------------------------------
for(i = 0; i < 16; i++)	lcd_putchar(STRINGSLcdHeader1[i]);
lcd_gotoxy(0,1);
for(i = 0; i < 16; i++)	lcd_putchar(STRINGSLcdHeader2[i]);

printf("\n\n\rRESET!\n\n\r");

delay_ms(3000);

INT_ON;		// enable interrupts

memset(gLineBuffer,0x00,sizeof(gLineBuffer));

printf("Suche 1-Wire Devices .... ");

// DS1820 am Bus suchen ...
devices = w1_search(0xf0,rom_code);

// ... und gefundene Menge anzeigen
printf("%d DS182x gefunden!\n\n\r",devices);

// ROM-Codes der gefundenen DS1820 anzeigen
if (devices)
	{
	for (i = 0; i < devices; i++)
		{
		printf("ROM-Code DS182x_%02d: ",i+1);

		for (j = 0; j < 8; j++) printf("%02X ",rom_code[i,j]);

		PCRLF;
		}
	PCRLF;
	}
else
	{
	printf("Keinen DS182x gefunden!\n\n\r");
	}

lcd_clear();

//---------------------------------------------------------------------------
// Start der Hauptschleife
//---------------------------------------------------------------------------
// Anschlussreihenfolge der Sensoren:
//
// Sensor 1 = ROM-Code 1 = INNEN
// Sensor 2 = ROM-Code 3 = VORLAUF
// Sensor 3 = ROM-Code 5 = Schlafzimmer
// Sensor 4 = ROM-Code 4 = Wohnzimmer
// Sensor 5 = ROM-Code 0 = Bad
// Sensor 6 = ROM-Code 2 = Kche
//---------------------------------------------------------------------------
while(1)
	{
//	sprintf(&gLineBuffer[0],"%03d,",counter);

	giMSecTimer = 0;	// reset 10 sec timer

	for(i = 0; i < devices; i++)
		{
		LED_ON;
		temp = ds1820_temperature_10(&rom_code[i,0]);
		LED_OFF;

		switch(i + 1)
			{
			case 1:
			// Sensor 5 = ROM-Code 0 = Bad
//			printf("%05d ",temp);
			if(temp == -9999)
				{
				sprintf(&gLineBuffer[20],"00.0,");	// 5
				sprintf(gLCDBuffer,"000",temp);
				}
			else
				{
				sprintf(&gLineBuffer[20],"%02i.%u,",temp/10,temp%10);	// 5
				sprintf(gLCDBuffer,"%03d,",temp);
				}
			lcd_gotoxy(6,1);
			break;

			case 2:
			// Sensor 1 = ROM-Code 1 = tRaum
//			printf("%05d ",temp);
			if(temp == -9999)
				{
				sprintf(&gLineBuffer[0],"00.0,");	// 5
				sprintf(gLCDBuffer,"000",temp);
				}
			else
				{
				sprintf(&gLineBuffer[0],"%02i.%u,",temp/10,temp%10);	// 5
				sprintf(gLCDBuffer,"%03d",temp);
				}
			lcd_gotoxy(0,0);
			break;

			case 3:
			// Sensor 6 = ROM-Code 2 = Kche
//			printf("%05d ",temp);
			if(temp == -9999)
				{
				sprintf(&gLineBuffer[25],"00.0,");	// 6
				sprintf(gLCDBuffer,"000",temp);
				}
			else
				{
				sprintf(&gLineBuffer[25],"%02i.%u,",temp/10,temp%10);	// 6
				sprintf(gLCDBuffer,"%03d",temp);
				}
			lcd_gotoxy(12,1);
			break;

			case 4:
			// Sensor 2 = ROM-Code 3 = Vorlauf
//			printf("%05d ",temp);
			if(temp == -9999)
				{
				sprintf(&gLineBuffer[5],"00.0,");	// 5
				sprintf(gLCDBuffer,"000",temp);
				}
			else
				{
				sprintf(&gLineBuffer[5],"%02i.%u,",temp/10,temp%10);	// 5
				sprintf(gLCDBuffer,"%03d",temp);
				}
			lcd_gotoxy(6,0);
			break;

			case 5:
			// Sensor 4 = ROM-Code 4 = Wohnzimmer
//			printf("%05d ",temp);
			if(temp == -9999)
				{
				tmp = gLineBuffer[20];
				sprintf(&gLineBuffer[15],"00.0,");	// 5
				gLineBuffer[20] = tmp;
				sprintf(gLCDBuffer,"000",temp);
				}
			else
				{
				tmp = gLineBuffer[20];
				sprintf(&gLineBuffer[15],"%02i.%u,",temp/10,temp%10);	// 5
				gLineBuffer[20] = tmp;
				sprintf(gLCDBuffer,"%03d",temp);
				}
			lcd_gotoxy(0,1);
			break;

			case 6:
			// Sensor 3 = ROM-Code 5 = Schlafzimmer
//			printf("%05d\n\r",temp);
			if(temp == -9999)
				{
				tmp = gLineBuffer[15];
				sprintf(&gLineBuffer[10],"00.0,");	// 5
				gLineBuffer[15] = tmp;
				sprintf(gLCDBuffer,"000",temp);
				}
			else
				{
				tmp = gLineBuffer[15];
				sprintf(&gLineBuffer[10],"%02i.%u,",temp/10,temp%10);	// 5
				gLineBuffer[15] = tmp;
				sprintf(gLCDBuffer,"%03d",temp);
				}
			lcd_gotoxy(12,0);
			break;
			}
		//-------------------------------------------------------------------
		// Nchsten LCD-Wert anzeigen
		//-------------------------------------------------------------------
		lcd_putchar(gLCDBuffer[0]);
		lcd_putchar(gLCDBuffer[1]);
		lcd_putchar('.');
		lcd_putchar(gLCDBuffer[2]);
		}

	//-----------------------------------------------------------------------
	// Pumpenrelais auswerten
	//-----------------------------------------------------------------------
	if(PINB.0)
		{
		sprintf(&gLineBuffer[30],"20.0,");
		lcd_gotoxy(4,0);
		lcd_putchar(SPC);
		lcd_putchar(SPC);
		lcd_gotoxy(10,0);
		lcd_putchar(SPC);
		lcd_putchar(SPC);
		}
	else
		{
		sprintf(&gLineBuffer[30],"40.0,");
		lcd_gotoxy(4,0);
		lcd_putchar('*');
		lcd_putchar('*');
		lcd_gotoxy(10,0);
		lcd_putchar('*');
		lcd_putchar('*');
		}

	//-----------------------------------------------------------------------
	// Analog Komparator auswerten (Brennerstart)
	//-----------------------------------------------------------------------
	if(ACSR & 0b00100000)
		{
		sprintf(&gLineBuffer[35],"39.8\n\r");
		lcd_gotoxy(4,1);
		lcd_putchar('*');
		lcd_putchar('*');
		lcd_gotoxy(10,1);
		lcd_putchar('*');
		lcd_putchar('*');
		}
	else
		{
		sprintf(&gLineBuffer[35],"25.0\n\r");
		lcd_gotoxy(4,1);
		lcd_putchar(SPC);
		lcd_putchar(SPC);
		lcd_gotoxy(10,1);
		lcd_putchar(SPC);
		lcd_putchar(SPC);
		}

	//-----------------------------------------------------------------------
	// String auf RS-232 ausgeben
	//-----------------------------------------------------------------------
	for(i = 0; i < sizeof(gLineBuffer); i++) putchar(gLineBuffer[i]);

//	counter++;

//	delay_ms(7000);	// alle 10 sec eine Messung

	while(giMSecTimer < 10000);
	}
}

/////////////////////////////////////////////////////////////////////////////
// Ende von ioctrl.c
/////////////////////////////////////////////////////////////////////////////
